import {
  useCallback, useContext, useEffect, useState,
} from 'react';
import { PropTypes } from 'prop-types';
import Button from 'react-bootstrap/Button';

import Web3 from 'web3';
import Web3Context from '../contexts/Web3Context';
import useList from '../utils/List';
import WorkContractContext from '../contexts/WorkContractContext';

const getRequestStatus = (request) => {
  switch (Number(request.status)) {
    case 0:
      return 'Requested';
    case 1:
      return 'Evidence Submitted';
    case 2:
      return 'Approval Submitted';
    case 3:
      return 'Complete';
    default:
      return 'Unknown';
  }
};

function WorkRequests({
  showRequestWork, verb,
}) {
  const [workRequests, dispatchWorkRequest] = useList();
  const [price, setPrice] = useState();
  const { workContract, availabilityStakes } = useContext(WorkContractContext);

  const {
    provider, account,
  } = useContext(Web3Context);

  const fetchPrice = useCallback(async () => {
    const web3 = new Web3(provider);
    const fetchedPrice = await workContract.methods.price().call();
    setPrice(web3.utils.fromWei(fetchedPrice, 'ether'));
  }, [workContract, provider]);

  const fetchWorkRequest = useCallback(async (requestIndex) => {
    const web3 = new Web3(provider);
    const r = await workContract.methods.requests(requestIndex).call();
    Object.assign(r, {
      id: Number(requestIndex),
      feeEther: web3.utils.fromWei(r.fee, 'ether'),
      currentUserIsCustomer: () => r.customer.toLowerCase()
            === account.toString().toLowerCase(),
    });
    dispatchWorkRequest({ type: 'update', item: r });
    return r;
  }, [workContract, provider, account, dispatchWorkRequest]);

  const fetchWorkRequests = useCallback(async () => {
    const count = await workContract.methods.requestCount().call();
    const promises = [];
    dispatchWorkRequest({ type: 'refresh' });
    for (let i = 0; i < count; i += 1) {
      promises.push(fetchWorkRequest(i));
    }
    await Promise.all(promises);
  }, [workContract, dispatchWorkRequest, fetchWorkRequest]);

  useEffect(() => {
    fetchPrice();
    fetchWorkRequests();

    workContract.events.WorkAssigned({ fromBlock: 'latest' }).on('data', (event) => {
      console.log('event: work assigned', event);
      fetchWorkRequest(event.returnValues.requestIndex);
    });

    workContract.events.WorkEvidenceSubmitted({ fromBlock: 'latest' }).on('data', (event) => {
      console.log('event: work evidence submitted', event);
      fetchWorkRequest(event.returnValues.requestIndex);
    });

    workContract.events.WorkApprovalSubmitted({ fromBlock: 'latest' }).on('data', (event) => {
      console.log('event: work approval submitted', event);
      fetchWorkRequest(event.returnValues.requestIndex);
    });
  }, [workContract, fetchWorkRequests, fetchPrice, fetchWorkRequest]);

  const requestWork = useCallback(async () => {
    // TODO: Accept input, upload to API, include hash in contract call
    const web3 = new Web3(provider);
    const priceWei = BigInt(web3.utils.toWei(price, 'ether'));
    await workContract.methods.requestWork('').send({
      from: account,
      gas: 1000000,
      value: priceWei,
    });
  }, [provider, workContract, account, price]);

  const submitWorkEvidence = useCallback(async (requestIndex) => {
    // TODO: Accept input, upload to API, include hash in contract call
    await workContract.methods.submitWorkEvidence(requestIndex, '').send({
      from: account,
      gas: 1000000,
    });
  }, [workContract, account]);

  const submitWorkApproval = useCallback(async (requestIndex) => {
    await workContract.methods.submitWorkApproval(requestIndex, true).send({
      from: account,
      gas: 1000000,
    });
  }, [workContract, account]);

  const submitWorkDisapproval = useCallback(async (requestIndex) => {
    await workContract.methods.submitWorkApproval(requestIndex, false).send({
      from: account,
      gas: 1000000,
    });
  }, [workContract, account]);

  return (
    <>
      <div>
        {`Price: ${price} ETH`}
      </div>
      {showRequestWork && (
        <div>
          <Button onClick={() => requestWork()}>{`Request ${verb}`}</Button>
        </div>
      )}
      <div>
        Request Count:
        {' '}
        {workRequests.length}
      </div>
      <div>
        <table className="table">
          <thead>
            <tr>
              <th>ID</th>
              <th>Customer</th>
              <th>Fee</th>
              <th>Status</th>
              <th>Stake ID</th>
              <th>Approval</th>
              {/* <th>Pool ID</th> */}
              <th>Actions</th>
            </tr>
          </thead>
          <tbody>
            {workRequests.filter((x) => !!x).map((request) => (
              <tr key={request.id}>
                <td>{request.id.toString()}</td>
                <td>{request.customer.toString()}</td>
                <td>
                  {request.feeEther}
                  {' '}
                  ETH
                </td>
                <td>{getRequestStatus(request)}</td>
                <td>{request.stakeIndex.toString()}</td>
                <td>{request.approval.toString()}</td>
                <td>
                  {availabilityStakes.length > 0
                    && availabilityStakes[Number(request.stakeIndex)]?.currentUserIsWorker()
                    && Number(request.status) === 0 && (
                    <Button onClick={() => submitWorkEvidence(request.id)}>
                      Submit Work Evidence
                    </Button>
                  )}
                  {request.currentUserIsCustomer()
                    && Number(request.status) === 1 && (
                      <>
                        <Button onClick={() => submitWorkApproval(request.id)}>
                          Submit Approval
                        </Button>
                        <Button onClick={() => submitWorkDisapproval(request.id)}>
                          Submit Dispproval
                        </Button>
                      </>
                  )}
                </td>
              </tr>
            ))}
          </tbody>
        </table>
      </div>
    </>
  );
}

WorkRequests.propTypes = {
  showRequestWork: PropTypes.bool,
  verb: PropTypes.string.isRequired,
};

WorkRequests.defaultProps = {
  showRequestWork: false,
};

export default WorkRequests;
